unit UDMSegReg;

interface

uses
  {Windows, Messages,} SysUtils, {Variants,} Classes, {Graphics,} Controls,
  Forms, Dialogs, UBaseForm, USegRegForm {StdCtrls, ComCtrls, StdActns, ActnList,
  Menus, ExtCtrls, ShellApi, Grids};

type DataArrayR = array of real;
     DataArrayI = array of integer;
     LineSet = array of string;
     RemoveSet = array of boolean;
type
  TDMSegReg = class(TDataModule)
    procedure DataModuleCreate(Sender: TObject);
  public
    FileName, Title1, Title2: string;
    SerialNr                : DataArrayI;
    Yvalue, Xvalue,
    Zvalue, Tvalue          : DataArrayR;
    Xdata,Ydata,Zdata       : DataArrayR;
    YrData,YcData           : DataArrayR;
    FileLine, OutLine       : LineSet;
    ToBeRemoved             : RemoveSet;
    NrOfData, TmpNrOfData,
    NrOfVar,  NrOfLInes,
    NrOfValues              : integer;
    Option, PrevOption      : byte;
    Code, Serial, Mark      : string[4];
    OneChar, XZcode         : char;
    Space                   : string[5];
    Stop                    : boolean;
    procedure ReadInputFile;
    procedure WriteInputFile (const DocName: string);
    procedure ReadXZcode;
    procedure ReadOutputLines(const DocName: string; var OutputOk : boolean;
                              const Extens:string);
    procedure ReadOutputData;
    procedure ProgramError(const Warning: string);
    function  Question(const Query: string): boolean;
  private
    { Private declarations }
  end;

var
  DMSegReg: TDMSegReg;


implementation

{$R *.dfm}

uses USegRegUtils, URegres1;


procedure TDMSegReg.DataModuleCreate(Sender: TObject);
var InitialDir: string;
begin
  getdir (0,InitialDir);
  FileName:=InitialDir;
end;


procedure TDMSegReg.ReadInputFile;
var  InFile: textfile;
     k: integer;
     HlpVar1 : string[33];
     HlpVar2 : string[22];
begin
  Assignfile(InFile, FileName);
  {$I-} Reset(InFile); {$I+}
  if ioresult <> 0 then
  begin
    Showmessage('The file "' + filename + '"could not be opened. It may be ' +
      'in use by another program,  or the'+
                ' programmer should incorporate more time.');
    exit;
  end;
  readln (InFile,Title1);
  readln (InFile,Title2);
  readln (inFile);
  read (InFile,HlpVar1);
  readln (InFile,NrOfVar);
  if NrOfVar=1 then Option:=1;
  read (InFile,HlpVar2);
  readln (InFile,NrOfData);
  SetLength(FileLine,6);
  for k:=1 to 6 do readln(InFile,FileLine[k-1]);
  k:=0;
  Try while not eof(InFile) do
      begin
        k:=k+1;
        SetLength(SerialNr,k);
        SetLength(Yvalue,k);
        SetLength(Xvalue,k);
        SetLength(ToBeRemoved,k);
        ToBeRemoved[k-1] := false;
        if NrOfVar=1 then
           readln(InFile,SerialNr[k-1],Yvalue[k-1],Xvalue[k-1])
        else
        begin
           SetLength(Zvalue,k);
           readln(InFile,SerialNr[k-1],Yvalue[k-1],Xvalue[k-1],Zvalue[k-1])
        end;
        NrOfData:=k;
      end;
  Except on E: exception
      do    ProgramError('Real numbers in the inputfile "' + filename +
                          '" are corrupt');
  end;
  Closefile(InFile);
end; {TDMSegReg.ReadInputFile}



procedure TDMSegReg.WriteInputFile (const DocName: string);
label 1;
var  InFile : textfile;
     k, m, TmpNrOfData: integer;
     Again : boolean;
begin
  Again := false;
  TmpNrOfData:=NrOfData;
1:Assignfile(InFile, FileName);
  {$I-} Rewrite(InFile); {$I+}
  if ioresult <> 0 then
  begin
    Showmessage('The file "' + filename + '"could not be opened. It may be ' +
      'in use by another program,  or the'+
                ' programmer should incorporate more time.');
    exit;
  end;
  writeln (InFile,Title1);
  writeln (InFile,Title2);
  writeln (InFile,' File name:  ',FileName);
  writeln (InFile,' Number of independent variables:  ',NrOfVar);
  writeln (InFile,' Number of data sets:  ',NrOfData);

  SetLength (FileLine,6);
  for k:=1 to 6 do FileLine[k-1]:='';
  FileLine[0] := ('');
    if NrOfVar=1 then
    begin
      FileLine[1] := ('');
      FileLine[2] :=
                ('          Dependent    independent');
      FileLine[3] :=
                ('Serial    variable     variable');
      FileLine[4] :=
                ('number    Y            X');
      FileLine[5] :=
                ('----------------------------------');
    end else
    begin
      FileLine[1] :=
              ('                       First          Second');
      FileLine[2] :=
              ('          Dependent    independent    independent');
      FileLine[3] :=
              ('Serial    variable     variable       variable');
      FileLine[4] :=
              ('number    Y            X              Z');
      FileLine[5] :=
              ('-------------------------------------------------');
    end;
  for k:=1 to 6 do writeln(InFile,FileLine[k-1]);

  m:=0;
  Try for k:=1 to DMSegReg.NrOfData do
      if TobeRemoved[k-1]=false then
      begin
        m:=m+1;
           if NrOfVar=1 then
             writeln(InFile, m:5,' ',Yvalue[k-1]:10:2,Xvalue[k-1]:13:2)
          else
             writeln(InFile, m:5,' ',Yvalue[k-1]:10:2,Xvalue[k-1]:13:2,
                                     Zvalue[k-1]:13:2)
      end else
          if not Again then NrOfData:=NrOfData-1;
    Except on E: exception
      do    ProgramError('Real numbers in the inputfile "' + filename +
                  '" are corrupt');
  end {Try for k:=1 to TmpNrOfData do};

  if not Again and (NrOfData<TmpNrOfData) then
  begin
    Closefile(InFile);
    Again := true;
    goto 1;
  end;
  Closefile(InFile);
end; {TDMSegReg.WriteInputFile}


procedure TDMSegReg.ReadXZcode;
var  OutFile: textfile;
     i: integer;
begin
  FileName := ChangeFileExt (DMSegReg.FileName,'.out');
  AssignFile(OutFile,FileName);
  {$I-} Reset(OutFile); {$I+}
  if ioresult <> 0 then
  begin
    Showmessage('The file "' + FileName + '"could not be opened. It may be ' +
      'damaged or in use by another program.');
    exit;
  end;
  for i:=1 to 20 do readln (OutFile);
  for i:=1 to 22 do
      read (OutFile,OneChar);
  read (OutFile,XZCode);
  CloseFile(OutFile);
end; {ReadXZcode}


procedure TDMSegReg.ReadOutputLines(const DocName: string;
                    var OutputOk : boolean; const Extens : string);
var  OutFile: textfile;
     k: integer;
begin
  AssignFile(OutFile,DocName);
  {$I-} Reset(OutFile); {$I+}
  if ioresult <> 0 then
  begin
    if Extens='.out' then
    Showmessage('The file "' + DocName + '"could not be opened. It may be ' +
      'damaged or in use by another program.')
    else
    Showmessage('The anova file "' + DocName + '"could not be found. Please ' +
      'go to the input tabsheet and run the program again to create it.');
    exit;
  end;
  k:=0;
  while not eof(OutFile) do
  begin
    k:=k+1;
    SetLength(OutLine,k);
    readln(OutFile,OutLine[k-1]);
  end;
  NrOfLines:=k;
  Close(OutFile);
  OutputOk := true;
  if NrOfLines<2 then
  begin
    OutputOk := false;
    deletefile (FileName);
    FileName := ChangeFileExt (FileName,'.var');
    deletefile (FileName);
    ShowMessage (' The output file is invalid due to inadequate' +
                 ' input data');
  end;
end; {TSegReg.ReadOutputLines}



procedure TDMSegReg.ReadOutputData;
var  OutFile: textfile;
     ii, Count, ConfLevel : integer;
     StDevX2: real;
     Failure : boolean;
     Str4, FirstBP : string[4];
     AllStr : string[43];
     T5Str  : string[35];
begin
  AssignFile(OutFile,FileName);
  {$I-} Reset(OutFile); {$I+}
  if ioresult <> 0 then
  begin
    Showmessage('The file "' + FileName + '"could not be opened. It may be ' +
      'damaged or in use by another program.');
    exit;
  end;

  for ii:=1 to 2 do readln (OutFile);
  readln (OutFile,Str4);
  Stop:=false;
  if Str4<>' The' then
  begin
    Stop:=true;
    showmessage ('This output file cannot be read by SegReg.'+
                 ' Graph will not be shown. Try SegRegA instead.');
    exit;
  end;

  Serial:='   ';
  while not (Serial=' SUM') do
        readln (OutFile,Serial);
  read (OutFile,Code);

  if Code=' Fun' then                                      {one indep. var}
  begin
    for ii:=1 to 12 do read (OutFile,OneChar);
    readln (OutFile,RegrType1);
    if RegrType1=2 then
    begin
      for ii:=1 to 16 do read (OutFile,OneChar);
      Failure:=false;
//The reading of RegrMethod is not strictly necessary when eliminated in URegr1
      Try readln (OutFile,RegrMeth1);
          Except on E: Exception do
          begin
            RegrMeth1:=0;
            Failure:=true;
          end;
      end;
      if not Failure then readln (OutFile);
      readln (OutFile);
      for ii:=1 to 32 do
          read (OutFile,OneChar);
      readln (OutFile,BP1);
    end;
    if RegrType1 in [3,4] then
    begin
      for ii:=1 to 16 do read (OutFile,OneChar);
      readln (OutFile,RegrMeth1);
    end;
    if RegrType1>2 then
    begin
      readln (OutFile);
      readln (OutFile);
      for ii:=1 to 32 do
          read (OutFile,OneChar);
      readln (OutFile,BP1);
    end;
  end;

  if Code<>' Fun' then                                    {two indep. var}
  begin
    for ii:=1 to 17 do read (OutFile,OneChar);
    readln (OutFile,RegrType1);
    for ii:=1 to 21 do read (OutFile,OneChar);
    readln (OutFile,RegrType2);
    if RegrType1 in [3,4]then
    begin
      for ii:=1 to 21 do read (OutFile,OneChar);
      readln (OutFile,RegrMeth1);
    end;
    if RegrType2 in [3,4]then
    begin
      for ii:=1 to 21 do read (OutFile,OneChar);
      readln (OutFile,RegrMeth2);
    end;
    while not (Serial=' (Se') do readln (OutFile,Serial);
    readln (OutFile);
    for ii:=1 to 40 do read (OutFile,OneChar);
    readln (OutFile,BP1);
    for ii:=1 to 40 do read (OutFile,OneChar);
    readln (OutFile,BP2);
  end;

  Reset (OutFile);

  if Code=' Fun' then                                      {one indep. var}
  begin
    while not (Serial=' Nam') do readln (OutFile,Serial);
    read (OutFile,Serial);
    if Serial=' Nam' then Count:=10
    else Count:=8;
    reset (OutFile);
    for ii:=1 to Count do readln (OutFile);
  end;
  if Code<>' Fun' then for ii:=1 to 14  do readln (OutFile);
  readln (OutFile,Title1);
  while pos ('  ',Title1)>0 do delete(Title1,Pos('  ',Title1),1);
  readln (OutFile,Title2);
  while pos ('  ',Title2)>0 do delete(Title2,Pos('  ',Title2),1);
  for ii:=1 to 23 do
      read (OutFile,OneChar);
  Try
    readln (OutFile,ConfLevel);
    Except on E: Exception do ConfLevel:=90;
  end;
  if ConfLevel<91 then Alfa:=1.75
  else
    if ConfLevel<96 then Alfa:=2.1
    else
      Alfa:=2.5;
  while not (Serial=' Reg') do readln (OutFile,Serial);
  for ii:=1 to 6 do readln (OutFile);
  read (OutFile,Mark);

  if Code=' Fun' then                                     {one indep. var}

  begin
    for ii:=1 to 20 do read (OutFile,OneChar);
    readln (OutFile,Nr1,AvY1,AvX1);
    readln (OutFile,Aux1,Aux1,StdA1);
    readln (OutFile,Aux1,StdY1,StDevX);
    AvXall := AvX1;
    StdAall := StdA1;
    StdYRall := StdY1;                                 {st.dev.Y residuals}
    RedSumXsq1:=Nr1*sqr(StDevX);
    if RegrType1>1 then
    begin
      while not (Serial=' Dat') do readln (OutFile,Serial);
      for ii:=1 to 20 do read (OutFile,OneChar);
      readln (OutFile,Nr1,AvY1,AvX1);
      readln (OutFile,Aux1,Aux1,StdA1);
      readln (OutFile,StDevY1,StdY1);     {St.dev.Y and st.dev.Y residuals <BP}
      readln (OutFile);
      for ii:=1 to 20 do read (OutFile,OneChar);
      readln (OutFile,Nr2,AvY2,AvX2);
      readln (OutFile,Aux2,Aux2,StdA2);
      readln (OutFile,StDevY2,StdY2);    {St.dev.Y and st.dev.Y residuals >BP}                      {st.dev.Y residuals}
      if (RegrType1=2) then
      begin
        for ii:=1 to 5 do readln (OutFile);
        readln (OutFile,YatBP,SDbp,ExplCoeff);
        ExplCoeffT:=ExplCoeff;
      end;
      if (RegrType1=3) or (RegrType1=4) then
      begin
        for ii:=1 to 3 do readln (OutFile);
        readln (OutFile,YatBP,YatBP);
        readln (OutFile);
        readln (OutFile,SDbp,SDbp);
        for ii:=1 to 3 do
            readln (OutFile,Serial);
        readln (OutFile,ExplCoeff,ExplCoeff,ExplCoeff);
        ExplCoeffT:=ExplCoeff;
      end;
      if (RegrType1>4) then
      begin
        for ii:=1 to 5 do readln (OutFile);
        readln (OutFile,T5Str,ExplCoeff);
        ExplCoeffT:=ExplCoeff;
      end;
    end;

  end;                                      {end of condition Code=Fun}

  if Code<>' Fun' then                      {two indep. var}

  begin
    reset (OutFile);
    Serial:='';
    while not (Serial=' Reg') do readln (OutFile,Serial);
    for ii:=1 to 6 do readln (OutFile);
    read (OutFile,FirstBP);
    for ii:=1 to 16 do read (OutFile,OneChar);
    readln (OutFile,Nr1,AvY1,AvX1);
    readln (OutFile,Aux1,Aux1,StdA1);
    readln (OutFile,Aux1,StdY1,StDevX);
    AvXall1 := AvX1;
    StdAall1 := StdA1;
    StdYRall1 := StdY1;                                   {st.dev.Y residuals}
    RedSumXsq1:=Nr1*sqr(StDevX);
    Serial:='';
    if RegrType1>1 then while not (Serial=' Par') do readln (OutFile,Serial)
    else while not (Serial=' Reg') do readln (OutFile,Serial);
    Serial:='';
    while not (Serial=' Reg') do readln (OutFile,Serial);
    for ii:=1 to 6 do readln (OutFile);
    for ii:=1 to 20 do read (OutFile,OneChar);
    readln (OutFile,Nr2,AvY2,AvX2);
    readln (OutFile,Aux1,Aux1,StdA2);
    readln (OutFile,Aux1,StdY2,StDevX2);
    AvXall2 := AvX2;
    StdAall2 := StdA2;
    StdYRall2 := StdY2;
    RedSumXsq2:=Nr2*sqr(StDevX2);
    reset (OutFile);
    if RegrType1>1 then
    begin
      while not (Serial=' Dat') do readln (OutFile,Serial);
      for ii:=1 to 20 do read (OutFile,OneChar);
      readln (OutFile,Nr1,AvY1,AvX1);
      readln (OutFile,Aux1,Aux1,StdA1);
      readln (OutFile,StDevY1,StdY1);  {St.dev.Y and st.dev.Y residuals <BP}
      readln (OutFile);
      for ii:=1 to 20 do read (OutFile,OneChar);
      readln (OutFile,Nr2,AvY2,AvX2);
      readln (OutFile,Aux2,Aux2,StdA2);
      readln (OutFile,StDevY2,StdY2);   {St.dev.Y and st.dev.Y residuals >BP}
      AvXall := (Nr1*AvX1+Nr2*AvX2)/(Nr1+Nr2);

      if (RegrType1=2) then
      begin
        for ii:=1 to 3 do readln (OutFile);
        readln (OutFile,SDbp,SDbp,RegrMeth1);
        readln (OutFile);
        readln (OutFile,YatBP,SDbp,ExplCoeff);
      end;
      if (RegrType1=3) or (RegrType1=4) then
      begin
        for ii:=1 to 3 do readln (OutFile);
        readln (OutFile,YatBP,YatBP);
        readln (OutFile);
        readln (OutFile,SDbp,SDbp);
        for ii:=1 to 3 do readln (OutFile);
        read (OutFile,ExplCoeff,ExplCoeff);
        Failure:=false;
        Try read (OutFile,ExplCoeff);
            Except on E: Exception do Failure:=true;
        end;
        readln (outFile);
        if Failure then
           readln (OutFile,ExplCoeff,ExplCoeff,ExplCoeff);
      end;
    end;

    serial := '    ';

    if FirstBP=' BPx' then
       while not (Serial=' BPz') do readln (OutFile,Serial)
    else
       while not (Serial=' BPx') do readln (OutFile,Serial);
    for ii:=1 to 24 do read (OutFile,OneChar);
    readln (OutFile,Corr);                      { Correlatie X,Z }

    for ii:=1 to 8 do readln (OutFile,Serial);
    for ii:=1 to 38 do read (OutFile,OneChar);
    readln (OutFile,AvYr1,AvXr1);
    readln (OutFile,Aux1,Aux1,StdB1);
    readln (OutFile,Aux1,StdYr1,StDevXR);
    if RegrType2>1 then
    begin
      while not (Serial=' Dat') do readln (OutFile,Serial);
      for ii:=1 to 20 do read (OutFile,OneChar);
      readln (OutFile,Nr1R,AvYr1,AvXr1);
      readln (OutFile,Aux1,Aux1,StdB1);
      readln (OutFile,StDevY1R,StdYr1); {St.dev.Y and st.dev.Y residuals <BP}
      readln (OutFile);
      for ii:=1 to 20 do read (OutFile,OneChar);
      readln (OutFile,Nr2R,AvYr2,AvXr2);
      readln (OutFile,Aux2,Aux2,StdB2);
      readln (OutFile,StDevY2R,StdYr2); {St.dev.Y and st.dev.Y residuals >BP}
      AvXallR := (Nr1R*AvXr1+Nr2R*AvXr2)/(Nr1R+Nr2R);

      if (RegrType2=2) then
      begin
        for ii:=1 to 3 do readln (OutFile);
        readln (OutFile,SDbp,SDbp,RegrMeth2);
        readln (OutFile);
        readln (OutFile,YatBPR,SDbpR,ExplCoeffR);
      end;
      if (RegrType2=3) or (RegrType2=4) then
      begin
        for ii:=1 to 3 do readln (OutFile);
        readln (OutFile,YatBPR,YatBPR);
        readln (OutFile);
        readln (OutFile,SDbpR,SDbpR);
        for ii:=1 to 3 do readln (OutFile);
        read (OutFile,ExplCoeffR,ExplCoeffR);
        Failure:=false;
        Try read (OutFile,ExplCoeffR);
            Except on E: Exception do Failure:=true;
        end;
        readln (outFile);
        if Failure then
           readln (OutFile,ExplCoeffR,ExplCoeffR,ExplCoeffR);
      end;
    end;
  end;                           { end of condition Code<>Fun}

  if Code=' Fun' then                   {reading RC and slope}
  begin
    if RegrType1<2 then
    begin
      while not (Serial=' The') do readln (OutFile,Serial);
      readln (OutFile);
      for ii:=1 to 10 do read (OutFile,OneChar);
      read (OutFile,AAs);
    end;

    if RegrType1>1 then
    begin
      while not (Serial=' whe') do readln (OutFile,Serial);
      readln (OutFile);
      readln (OutFile);
      for ii:=1 to 10 do read (OutFile,OneChar);
      read (OutFile,AAs);
      while not (OneChar='A') do read (OutFile,OneChar);
      for ii:=1 to 4 do read (OutFile,OneChar);
      readln (OutFile,Ag);
    end;
  end;                                      {end of condition Code=Fun }

  if Code<>' Fun' then                      {reading RC and slopes}
  begin
     while not (Serial=' whe') do readln (OutFile,Serial);
     for ii:=1 to 4 do readln (OutFile);
     for ii:=1 to 11 do read (OutFile,OneChar);
     read (OutFile,AAs);
     while not (OneChar='B') do read (OutFile,OneChar);
     for ii:=1 to 4 do read (OutFile,OneChar);
     readln (OutFile,Bs);
     for ii:=1 to 11 do read (OutFile,OneChar);
     read (OutFile,Ag);
     while not (OneChar='B') do read (OutFile,OneChar);
     for ii:=1 to 4 do read (OutFile,OneChar);
     readln (OutFile,Bg);
     for ii:=1 to 3 do readln (OutFile);
     readln (Outfile,AllStr,ExplcoeffT);
  end;                                      { end of condition Code<>Fun }

  while not (Serial=' ---') do readln (OutFile,Serial);
  ii := 0;

  if Code=' Fun' then
  begin
    while not eof (OutFile) do
    begin
      ii := ii+1;
      SetLength (Xdata,ii);
      SetLength (Ydata,ii);
      SetLength (Ycdata,ii);
      readln (OutFile,Serial,Space,Ydata[ii-1],Xdata[ii-1],YcData[ii-1]);
    end;
    DMSegReg.NrOfData := ii;
  end;

  if Code= ' Y-X' then
  begin
    while not eof (OutFile) do
    begin
      ii := ii+1;
      SetLength (Xdata,ii);
      SetLength (Ydata,ii);
      SetLength (Yrdata,ii);
      SetLength (Zdata,ii);
      SetLength (Ycdata,ii);
      readln
        (OutFile,Serial,Space,Ydata[ii-1],Xdata[ii-1],YrData[ii-1],
                              Zdata[ii-1],YcData[ii-1]);
    end;
    DMSegreg.NrOfData := ii;
  end;

  if Code= ' Y-Z' then
  begin
    while not eof (OutFile) do
    begin
      ii := ii+1;
      SetLength (Xdata,ii);
      SetLength (Ydata,ii);
      SetLength (Yrdata,ii);
      SetLength (Zdata,ii);
      SetLength (Ycdata,ii);
      readln
        (OutFile,Serial,Space,Ydata[ii-1],Zdata[ii-1],YrData[ii-1],
                              Xdata[ii-1],YcData[ii-1]);
    end;
    DMSegreg.NrOfData := ii;
  end;
  CloseFile(OutFile);
end; {TDMSegReg.ReadOutputData}



procedure TDMSegReg.ProgramError(const Warning: string);
begin Showmessage('A program error occurred.' + Warning);
      Application.Terminate;
end; {TDMSegReg.ProgramError}



function TDMSegReg.Question(const Query: string): boolean;
begin Result := MessageDlg(Query, mtConfirmation, [mbYes, mbNo], 0) = mrYes;
end; {TDMEnDrain.Question}


end.


